from .antd.button import button  # noqa
from .antd.icon import icon  # noqa
from .antd.float_button import float_button, float_group, float_back_top  # noqa
from .antd.typography import typography_text, typography_title, typography_paragraph  # noqa
from .antd.affix import affix  # noqa




from typing import Optional, Union

from reflex import Component, Var
from reflex.utils import imports

from ..base import AntdComponent
from ..constant import OrientationType, DirectionType


class Flex(AntdComponent):
    tag = 'Flex'

    vertical: Optional[Var[bool]]
    wrap: Optional[Var[str]]
    justify: Optional[Var[str]]
    align: Optional[Var[str]]
    flex: Optional[Var[str]]
    gap: Optional[Var[str]]
    component: Optional[Var[str]]


flex = Flex.create




from typing import Optional, Union, Dict, Any, List
from reflex import Var, Component
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import StatusType, SizeType, SelectModeType, PlacementType, VariantType


class Switch(AntdComponent):
    tag = 'Switch'

    auto_focus: Optional[Var[bool]]
    checked: Optional[Var[bool]]
    checked_children: Optional[Var[ReactNode]]
    default_checked: Optional[Var[bool]]
    default_value: Optional[Var[bool]]
    disabled: Optional[Var[bool]]
    loading: Optional[Var[bool]]
    size: Optional[Var[SizeType]]
    un_checked_children: Optional[Var[ReactNode]]
    value: Optional[Var[bool]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CHANGE: lambda checked, e: [checked, e],
            EventTriggers.ON_CLICK: lambda checked, e: [checked, e],
        })
        return _triggers


switch = Switch.create




from typing import Optional, Union, Dict, Any

from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import VariantType, SizeType


class AutoComplete(AntdComponent):
    tag = "AutoComplete"

    allow_clear: Optional[Var[bool]]
    auto_focus: Optional[Var[bool]]
    backfill: Optional[Var[bool]]
    default_active_first_option: Optional[Var[bool]]
    default_open: Optional[Var[bool]]
    default_value: Optional[Var[str]]
    disabled: Optional[Var[bool]]
    popup_class_name: Optional[Var[str]]
    dropdown_match_select_width: Optional[Var[Union[bool, int]]]
    filter_option: Optional[Var[bool]]
    not_found_content: Optional[Var[ReactNode]]
    open: Optional[Var[bool]]
    options: Optional[Var[list]]
    placeholder: Optional[Var[str]]
    status: Optional[Var[str]]
    size: Optional[Var[SizeType]]
    value: Optional[Var[str]]
    variant: Optional[Var[VariantType]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_BLUR: lambda: [],
            "on_dropdown_visible_change": lambda open: [open],
            EventTriggers.ON_FOCUS: lambda: [],
            "on_search": lambda value: [value],
            EventTriggers.ON_SELECT: lambda value, option: [value, option],
            "on_clear": lambda: [],
        })
        return _triggers


auto_complete = AutoComplete.create



from typing import Any, Dict, Literal, Optional, Union, overload
from reflex.vars import Var, BaseVar, ComputedVar
from reflex.event import EventChain, EventHandler, EventSpec
from reflex.style import Style
from typing import Optional, Union, Dict, Any, List, Tuple
from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers
from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import SizeType, PlacementType, TabsType

class StatisticBase(AntdComponent):

    @overload
    @classmethod
    def create(cls, *children, prefix: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, suffix: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, title: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, value: Optional[Union[Var[Union[str, int]], Union[str, int]]]=None, value_style: Optional[Union[Var[Dict], Dict]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'StatisticBase':
        """Create the component.

        Args:
            *children: The children of the component.
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...

class Statistic(StatisticBase):

    @overload
    @classmethod
    def create(cls, *children, decimal_separator: Optional[Union[Var[str], str]]=None, formatter: Optional[Union[Var[JsValue], JsValue]]=None, group_separator: Optional[Union[Var[str], str]]=None, loading: Optional[Union[Var[bool], bool]]=None, precision: Optional[Union[Var[int], int]]=None, prefix: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, suffix: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, title: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, value: Optional[Union[Var[Union[str, int]], Union[str, int]]]=None, value_style: Optional[Union[Var[Dict], Dict]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'Statistic':
        """Create the component.

        Args:
            *children: The children of the component.
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...

class StatisticCountdown(StatisticBase):

    def get_event_triggers(self) -> Dict[str, Any]:
        ...

    @overload
    @classmethod
    def create(cls, *children, format: Optional[Union[Var[str], str]]=None, prefix: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, suffix: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, title: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, value: Optional[Union[Var[Union[str, int]], Union[str, int]]]=None, value_style: Optional[Union[Var[Dict], Dict]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_finish: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'StatisticCountdown':
        """Create the component.

        Args:
            *children: The children of the component.
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...
statistic = Statistic.create
statistic_countdown = StatisticCountdown.create


from typing import Optional, Union, Dict, Any

from reflex import Var, Component
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, ReactNode
from ..constant import TypeType, ButtonShape, SizeType


class Button(AntdComponent):
    tag = "Button"

    block: Optional[Var[bool]]
    danger: Optional[Var[bool]]
    disabled: Optional[Var[bool]]
    ghost: Optional[Var[bool]]
    href: Optional[Var[str]]
    target: Optional[Var[str]]
    html_type: Optional[Var[str]]
    icon: Optional[Var[ReactNode]]
    loading: Optional[Var[bool]]
    shape: Optional[Var[ButtonShape]]
    size: Optional[Var[SizeType]]
    type: Optional[Var[TypeType]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()

        _triggers.update({
            EventTriggers.ON_CLICK: lambda: [],
        })
        return _triggers


button = Button.create



from typing import Optional, Union, Dict, Any, List, Tuple

from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import TimelineModeType


class Tag(AntdComponent):
    tag = 'Tag'

    close_icon: Optional[Var[Union[bool, ReactNode]]]
    color: Optional[Var[str]]
    icon: Optional[Var[ReactNode]]
    bordered: Optional[Var[bool]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            'on_close': lambda e: [e],
        })
        return _triggers


class CheckableTag(AntdComponent):
    tag = 'Tag.CheckableTag'

    checked: Optional[Var[bool]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CHANGE: lambda checked: [checked],
        })
        return _triggers


tag = Tag.create
checkable_tag = CheckableTag.create




from typing import Optional, Union, Dict, Any
from reflex import Var, Component
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import StatusType, SizeType, VariantType


class InputNumber(AntdComponent):
    tag = "InputNumber"

    addon_after: Optional[Var[ReactNode]]
    addon_before: Optional[Var[ReactNode]]
    auto_focus: Optional[Var[bool]]
    change_on_blur: Optional[Var[bool]]
    change_on_wheel: Optional[Var[bool]]
    controls: Optional[Var[Union[bool, ContainVar]]]
    decimal_separator: Optional[Var[str]]
    placeholder: Optional[Var[str]]
    default_value: Optional[Var[int]]
    disabled: Optional[Var[bool]]
    formatter: Optional[Var[JsValue]]
    keyboard: Optional[Var[bool]]
    max: Optional[Var[int]]
    min: Optional[Var[int]]
    parser: Optional[Var[JsValue]]
    precision: Optional[Var[int]]
    read_only: Optional[Var[bool]]
    status: Optional[Var[StatusType]]
    prefix: Optional[Var[ReactNode]]
    size: Optional[Var[SizeType]]
    step: Optional[Var[Union[int, str]]]
    string_mode: Optional[Var[bool]]
    value: Optional[Var[int]]
    variant: Optional[Var[VariantType]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CHANGE: lambda value: [value],
            'on_press_enter': lambda e: [e],
            "on_step": lambda value, info: [value, info],
        })
        return _triggers


input_number = InputNumber.create



from typing import Optional, Union, Dict, Any, List, Set, Iterator
import uuid

from reflex import Component, Var
from reflex.utils import imports
from reflex.components.base.bare import Bare
from reflex.vars import BaseVar

from ..base import AntdComponent, ContainVar, JsValue, ReactNode, ExStateItem, version
from ..constant import MessageType
from . import helper


class Message(JsValue):
    is_global: bool = True
    class_name: Optional[Var[str]]
    content: Optional[Var[ReactNode]]
    duration: Optional[Var[Union[int, float]]]
    icon: Optional[Var[ReactNode]]
    key: Optional[Var[Union[str, int]]]
    style: Optional[Var[dict]]
    type: Optional[MessageType] = 'info'
    config: Optional[Var[dict]]

    top: Optional[Var[int]]
    rtl: Optional[Var[bool]]
    prefix_cls: Optional[Var[str]]
    max_count: Optional[Var[int]]

    @property
    def config_item(self) -> Optional[ExStateItem]:
        if not hasattr(self, 'config') or self.config is None:
            return None
        try:
            return self._config_item
        except AttributeError:
            self._config_item = ExStateItem(self.config, self)
            return self._config_item

    @property
    def uid(self) -> str:
        try:
            return self._uid
        except AttributeError:
            self._uid = uuid.uuid4().hex
            return self._uid

    def get_open_message(self) -> str:
        return f'openMessage_{self.uid}'

    def get_imports(self) -> imports.ImportDict:
        _imports = {
            "antd": [imports.ImportVar(tag='message')],
            "react": [imports.ImportVar(tag="useEffect")],
        }
        return _imports

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        if self.config_item is not None:
            open_func = """const %(name)s = () => {
                           %(mn)s.open(%(cfg)s);
                       };""" % dict(
                name=self.get_open_message(),
                mn='messageApi' if not self.is_global else 'message',
                cfg=self.config_item.serialize(),
            )
        else:
            key_s = f'{self.key},' if hasattr(self, 'key') else ''
            duration_s = f'duration: {self.duration},' if hasattr(self, 'duration') else ''
            top_s = f'top: {self.top},' if hasattr(self, 'top') else ''

            open_func = """
            const %(name)s = () => {
                %(mn)s.open({
                  %(key)s
                  type: '%(type)s',
                  content: '%(content)s',
                  %(duration)s %(top)s
                });
            };
            """ % dict(
                name=self.get_open_message(),
                mn='messageApi' if not self.is_global else 'message',
                key=key_s, type=self.type, content=self.content,
                duration=duration_s,
                top=top_s,
            )
        others = []
        if self.config_item:
            state = self.config_item.serialize()
            others.extend([
                *self.config_item.get_hooks(),
                helper.hook_state(
                    state_field=state,
                    on_update='%(name)s(%(state)s)' % dict(name=self.get_open_message(), state=state),
                    on_null='messageApi.destroy()',
                ),
            ])

        _hooks = []
        if self.is_global:
            _hooks.extend([
                open_func
            ])
        else:
            _hooks.extend([
                """const [messageApi, contextHolder] = message.useMessage();""",
                open_func,
                *others,
            ])
        if version <= '000.004.006':
            return set(_hooks)
        return dict((h, None) for h in _hooks)

    def serialize(self) -> str:
        # if self.config_item is not None:
        #     return '{contextHolder}'
        return self.get_open_message()


class MessageHolder(Bare):
    """
    Message.hook:
        const [messageApi, contextHolder] = message.useMessage();

    MessageHolder:
        <>
            {contextHolder}
        </>
     """
    msg: Optional[Message]

    @classmethod
    def create(cls, msg: Message) -> Component:
        cs = ['<>']
        if not msg.is_global:
            cs.append('{contextHolder}')
        cs.append('</>')
        return cls(
            msg=msg,
            contents='\n'.join(cs),
        )

    def _get_vars(self, include_children: bool = False) -> Iterator[Var]:
        yield self.contents
        yield BaseVar(
            _var_name='',
            _var_is_local=True,
            _var_data=self.msg.get_var_data(),
        )


message = Message
message_holder = MessageHolder.create

# def message_holder():
#     return Var.create_safe('{contextHolder}')



from typing import Optional, Union, Dict, Any

from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue
from ..constant import PlacementType, DirectionType, SizeType, StepsStatusType, StepsType


class Steps(AntdComponent):
    tag = 'Steps'

    current: Optional[Var[int]]
    direction: Optional[Var[DirectionType]]
    initial: Optional[Var[int]]
    label_placement: Optional[Var[DirectionType]]
    percent: Optional[Var[int]]
    progress_dot: Optional[Var[Union[bool, JsValue]]]
    responsive: Optional[Var[bool]]
    size: Optional[Var[SizeType]]
    status: Optional[Var[StepsStatusType]]
    type: Optional[Var[StepsType]]

    items: Optional[Var[ContainVar]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()

        _triggers.update({
            EventTriggers.ON_CHANGE: lambda current: [current],
        })
        return _triggers


steps = Steps.create





from typing import Optional, Union, Dict, Any

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import SizeType


class AList(AntdComponent):
    tag = 'List'

    bordered: Optional[Var[bool]]
    data_source: Optional[Var[Any]]
    footer: Optional[Var[ReactNode]]
    grid: Optional[Var[dict]]
    header: Optional[Var[ReactNode]]
    item_layout: Optional[Var[str]]
    loading: Optional[Var[Union[bool, dict]]]
    load_more: Optional[Var[ReactNode]]
    locale: Optional[Var[dict]]
    pagination: Optional[Var[Union[bool, dict]]]
    render_item: Optional[Var[JsValue]]
    size: Optional[Var[SizeType]]
    split: Optional[Var[bool]]


class ListItem(AntdComponent):
    tag = 'List.Item'

    actions: Optional[Var[list[ContainVar]]]
    extra: Optional[Var[ReactNode]]


class ListItemMeta(AntdComponent):
    tag = 'List.Item.Meta'

    avatar: Optional[Var[ReactNode]]
    description: Optional[Var[ReactNode]]
    title: Optional[Var[ReactNode]]


alist = AList.create
list_item = ListItem.create
list_item_meta = ListItemMeta.create



from typing import Type, Any, Tuple, Dict, List, Iterable, Callable, Set, Union, Optional, Self
import sys
from os import path
from abc import ABC, abstractmethod
from functools import lru_cache, wraps
from hashlib import md5
import uuid
import dataclasses
import inspect
import re

import reflex as rx
from reflex import Component, Var, State, Base
from reflex.base import pydantic
from reflex.components.component import BaseComponent, CustomComponent, StatefulComponent
from reflex.components.base.bare import Bare
from reflex.constants import Hooks, Reflex, MemoizationDisposition, MemoizationMode
from reflex.utils import imports, format
from reflex.vars import BaseVar, VarData
from reflex.event import EventHandler, EventSpec, EventChain

from .constant import SizeType
from .util import OrderedSet

# 0.4.6 -> 000.004.006
version = '.'.join(map(lambda x: x.zfill(3), Reflex.VERSION.split('.')))

my_path = path.abspath(path.dirname(__file__))
template_path = path.join(my_path, '.templates')

APP_ROUTER = False
RE_KEY_IDX = re.compile(r'\.\d+\.')

memo_never = MemoizationMode().set(disposition=MemoizationDisposition.NEVER)
memo_never_no_recursive = MemoizationMode().set(disposition=MemoizationDisposition.NEVER, recursive=False)
memo_always = MemoizationMode().set(disposition=MemoizationDisposition.ALWAYS)
memo_always_no_recursive = MemoizationMode().set(disposition=MemoizationDisposition.ALWAYS, recursive=False)


def stateful(hd: Callable[..., Component] = None, forced=True) -> Callable:
    """ render a component into a function """

    def _my(_hd: Callable[..., Component]) -> Callable:
        @wraps(_hd)
        def _wrap(*args, **kwargs):
            _com = _hd(*args, **kwargs)
            if forced:
                _com._memoization_mode = memo_always
                # _com._memoization_mode.recursive = False
            # _sc = StatefulComponent.create(_com)
            # return _sc if _sc is not None else _com
            return _com
        return _wrap

    if hd is None:
        return _my
    else:
        return _my(hd)


def pretty_dumps(value: Any, indent=2) -> str:
    return format.json.dumps(value, ensure_ascii=False, default=format.serialize, indent=indent)


def get_component_all_imports(com: Component) -> imports.ImportDict:
    if version <= '000.004.006':
        return {} if isinstance(com, str) else com.get_imports()
    else:
        return {} if isinstance(com, str) else com._get_all_imports()


def get_component_hooks(com) -> Set[str] | Dict[str, None]:
    if version <= '000.004.006':
        return set() if isinstance(com, str) else com.get_hooks()
    return set() if isinstance(com, str) \
        else com._get_all_hooks_internal() | com._get_all_hooks()


def get_component_custom_code(com: Component) -> Set[str]:
    if version <= '000.004.006':
        return com.get_custom_code() if isinstance(com, BaseComponent) else set()
    return com._get_all_custom_code() if isinstance(com, BaseComponent) else set()


class ExItem(ABC):
    @classmethod
    def isinstance(cls, item: Any) -> bool:
        pass

    def __init__(self, item: Any, parent: Component, key: str = ''):
        self.item = item
        self.key = key
        self.parent = parent

    @abstractmethod
    def serialize(self) -> str:
        ...

    def get_imports(self) -> imports.ImportDict:
        return {}

    def get_state(self) -> str:
        return ''

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        if version <= '000.004.006':
            return set()
        return {}

    def get_interpolations(self) -> List[Tuple[int, int]]:
        return []

    def get_var_data(self) -> VarData:
        return VarData(
            state=self.get_state(),
            imports=self.get_imports(),
            hooks=self.get_hooks(),
            interpolations=self.get_interpolations(),
        )

    def get_custom_components(self) -> set[CustomComponent]:
        return set()

    def get_custom_code(self) -> set[str]:
        return set()


class ExComponentItemBase(ExItem):
    item: Component

    @classmethod
    def isinstance(cls, item: Any) -> bool:
        return isinstance(item, Component)

    def serialize(self) -> str:
        return str(self.item)

    def get_imports(self) -> imports.ImportDict:
        return get_component_all_imports(self.item)

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return get_component_hooks(self.item)

    def get_custom_components(self) -> set[CustomComponent]:
        return {self.item} if isinstance(self.item, CustomComponent) else set()

    def get_custom_code(self) -> set[str]:
        return get_component_custom_code(self.item)


class ExComponentItem(ExComponentItemBase):
    item: Component


class ExStatefulComponentItem(ExComponentItemBase):
    item: StatefulComponent

    @classmethod
    def isinstance(cls, item: Any) -> bool:
        return isinstance(item, StatefulComponent)

    def serialize(self) -> str:
        return f'<{self.item.tag}/>'


class JsEvent:
    hd: EventHandler
    # event_trigger use for custom trigger, must work with fmt=True
    event_trigger: Callable

    def __init__(self, hd: Any, js: str = '', fmt: bool = False, event_trigger: Callable = None):
        assert isinstance(hd, EventHandler)
        self.hd = hd
        self.js = js
        self.fmt = fmt
        self.event_trigger = event_trigger

    def get_state_full_name(self) -> str:
        name = str(self.hd.fn).split(' ')[1]
        _base = format.to_snake_case(name)
        return f'state.{_base}'

    def get_event_args(self) -> str:
        _args = inspect.getfullargspec(self.hd.fn)
        rs = []
        for idx, _arg in enumerate(_args.args):
            if idx == 0:  # ignore self
                continue
            rs.append(f'{_arg}:{self.args[idx-1]}')
        return ','.join(rs)

    def get_ex_item(self, parent, key) -> ExItem:
        item = ExEventHandlerItem(self, parent, key=key)
        return item


js_event = JsEvent


class ExEventHandlerItem(ExItem):
    item: JsEvent

    @classmethod
    def isinstance(cls, item: Any) -> bool:
        return isinstance(item, JsEvent)

    @property
    def hd_item(self) -> 'ExLambdaHandlerItem':
        try:
            return self._hd_item
        except AttributeError:
            self._hd_item = ExLambdaHandlerItem(self.item.hd, self.parent, key=self.key)
            self._hd_item.event_trigger = self.item.event_trigger
            return self._hd_item

    def _get_fn_name(self) -> str:
        return f"{self.key.replace('.', '_')}_{md5(f'{str(self.item.hd)}'.encode('utf-8')).hexdigest()}_{id(self)}"

    def serialize(self) -> str:
        hd_name = self.hd_item.serialize()
        trigger = self.hd_item._get_event_trigger()
        args = inspect.getfullargspec(trigger).args

        if not self.item.fmt:
            return f"""({','.join(args)}) => {{
                {self.item.js}
                {hd_name}({','.join(args)})
            }}
            """
        else:
            return self.item.js % dict(name=hd_name)

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return self.hd_item.get_hooks()

    def get_imports(self) -> imports.ImportDict:
        return self.hd_item.get_imports()


class ExLambdaHandlerItem(ExItem):
    item: Callable
    event_trigger: Callable = None

    @classmethod
    def isinstance(cls, item: Any) -> bool:
        return isinstance(item, Callable)

    def _get_fn_name(self) -> str:
        return f"{self.key.replace('.', '_')}_{md5(f'{str(self.item)}'.encode('utf-8')).hexdigest()}_{id(self)}"

    def _get_event_trigger_key(self) -> str:
        return RE_KEY_IDX.sub('.*.', self.key)

    def _get_event_trigger(self) -> Callable:
        if self.event_trigger is not None:
            return self.event_trigger
        triggers = self.parent.get_event_triggers()
        rs = triggers.get(self._get_event_trigger_key(), lambda: [])
        return rs

    def serialize(self) -> str:
        return self._get_fn_name()

    def get_imports(self) -> imports.ImportDict:
        return {"react": [imports.ImportVar(tag="useCallback")]}

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        if version <= '000.004.005':
            key = self._get_event_trigger_key()
            chain = self.parent._create_event_chain(key, self.item)
        else:
            key = self._get_event_trigger()
            chain = self.parent._create_event_chain(key, self.item)
        rendered_chain = format.format_prop(chain).strip("{}")
        _hook = f"""const {self._get_fn_name()} = useCallback({rendered_chain}, [addEvents, Event]);"""

        if version <= '000.004.006':
            return {_hook}
        return {_hook: None}


class ExCallableItem(ExItem):
    item: Callable

    @classmethod
    def isinstance(cls, item: Any) -> bool:
        return isinstance(item, Callable)

    def serialize(self) -> str:
        return str(self.item())

    def get_imports(self) -> imports.ImportDict:
        return get_component_all_imports(self.item())

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return get_component_hooks(self.item())


class ExStateItem(ExItem):
    item: BaseVar

    @classmethod
    def isinstance(cls, item: Any) -> bool:
        return isinstance(item, BaseVar) and item._var_full_name.startswith("state__")

    def serialize(self) -> str:
        return self.item._var_full_name

    def get_imports(self) -> imports.ImportDict:
        return self.item._var_data.imports

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return self.item._var_data.hooks

    def get_state(self) -> str:
        return self.item._var_data.state


class FakeComponentMixin:
    def _create_event_chain(self, *args, **kwargs):
        return Component._create_event_chain(self, *args, **kwargs)

    def get_event_triggers(self) -> Dict[str, Any]:
        return {}

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return {}

    def get_imports(self) -> imports.ImportDict:
        return {}


class JsValue:
    """
    Component: support rx.foreach...,
      <Breadcrumb items={(state__global_state.subnav_items.map((i, index_de38f21f196b8746f2b4ca2b3fd6b46b) => (
        {
          "title": <RadixThemesLink asChild={true}>
              <NextLink href={i["href"]} passHref={true}>
              {i["title"]}
              </NextLink>
          </RadixThemesLink>
        }
          )))}
      />
    """
    value: Callable | str | Component

    def __init__(self, value: Union[str, Callable, Component] = None, **kwargs):
        self.value = value
        self._init(**kwargs)

    def _init(self, **kwargs) -> None:
        for k, v in kwargs.items():
            setattr(self, k, v)

    def get_ex_item(self, parent, key) -> ExItem:
        item = ExJsItem(self, parent, key=key)
        return item

    def serialize(self) -> str:
        return f"({self.value if isinstance(self.value, str) else str(self.value).strip('{}')})"

    def get_imports(self) -> imports.ImportDict:
        return get_component_all_imports(self.value)

    def get_state(self) -> str:
        return ''

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return get_component_hooks(self.value)

    def get_var_data(self) -> VarData:
        return VarData(
            state=self.get_state(),
            imports=self.get_imports(),
            hooks=self.get_hooks(),
        )

    def get_custom_components(self) -> set[CustomComponent]:
        return get_component_custom_code(self.value)


class JsFunctionValue(JsValue):
    _value: Union[str, Component]

    def _init(self, **kwargs) -> None:
        super()._init(**kwargs)
        self._args = inspect.getfullargspec(self.value)
        args = self._args.args
        self._value = self.value(*args)

    def serialize(self) -> str:
        is_component = False
        if isinstance(self._value, str):
            v = self._value
        elif isinstance(self._value, Component):
            is_component = True
            v = str(self._value)
        else:
            raise TypeError(self._value)
        sep_1, sep_2 = ('{{', '}}') if not is_component else ('(', ')')
        return f"""(({','.join(self._args.args)}) => 
        {sep_1} {v} {sep_2} )"""

    def get_imports(self) -> imports.ImportDict:
        return get_component_all_imports(self._value)

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return get_component_hooks(self._value)

    def get_custom_components(self) -> set[CustomComponent]:
        return set() if not isinstance(self._value, CustomComponent) else {self._value}


def js_value(value: Union[str, Callable, Component], **kwargs) -> JsValue:
    if isinstance(value, (str, Component)):
        return JsValue(value, **kwargs)
    if isinstance(value, Callable):
        return JsFunctionValue(value, **kwargs)
    raise NotImplemented("Not implemented: %s(%s)" % (type(value), value))


class ExJsItem(ExItem):
    item: JsValue

    @classmethod
    def isinstance(cls, item: Any) -> bool:
        return isinstance(item, JsValue)

    def serialize(self) -> str:
        return self.item.serialize()

    def get_imports(self) -> imports.ImportDict:
        return self.item.get_imports()

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return self.item.get_hooks()

    def get_custom_components(self) -> set[CustomComponent]:
        return self.item.get_custom_components()


class ExVarItem(ExItem):
    item: Var

    @classmethod
    def isinstance(cls, item: Any) -> bool:
        return isinstance(item, Var)

    def serialize(self) -> str:
        return str(self.item).strip('{}')

    def get_imports(self) -> imports.ImportDict:
        return self.item._var_data.imports if self.item._var_data else {}

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return self.item._var_data.hooks if self.item._var_data else {}

    def get_state(self) -> str:
        return self.item._var_data.state if self.item._var_data else ""

    def get_interpolations(self) -> List[Tuple[int, int]]:
        return self.item._var_data.interpolations if self.item._var_data else []


class ExFormatter:
    items: List[Type[ExItem]] = [
        ExVarItem,
        ExStatefulComponentItem,
        ExComponentItem,
        ExStateItem,
        ExJsItem,
        ExEventHandlerItem,
        ExLambdaHandlerItem,
        # ExCallableItem,
    ]

    def __init__(self, value: Any, parent: Component, name: str = ''):
        self.name = name
        self.value = value
        self.parent = parent
        self._coms: Dict[str, ExItem] = {}
        self._value: Any = None
        self._extract()

    def _extract(self):
        value = self.value
        self._coms.clear()

        def _op(_v: Any, key: str):
            if isinstance(_v, (list, tuple)):
                return _list(_v, pkey=key)
            elif isinstance(_v, dict):
                return _dict(_v, pkey=key)
            elif isinstance(_v, (int, float, str, bool)):
                return _v

            _id = _v
            for ex in self.items:
                if ex.isinstance(_v):
                    _id = uuid.uuid4().hex
                    self._coms[_id] = ex(_v, self.parent, key=key)
                    return _id

            if isinstance(_v, Base):
                _d = dict(_v._iter(exclude_unset=True, exclude_none=True, to_dict=False))
                return _dict(_d, pkey=key)
            elif dataclasses.is_dataclass(_v):
                _d = dataclasses.asdict(_v)
                return _dict(_d, pkey=key)

            return _v

        def _list(_v: List, pkey: str) -> List[Any]:
            return [_op(i, key=f'{pkey}.{idx}') for idx, i in enumerate(_v)]

        def _dict(_v: Dict, pkey: str) -> Dict[str, Any]:
            return dict((format.to_camel_case(k), _op(v, key=f'{pkey}.{k}')) for k, v in _v.items() if v is not None)

        rs = _op(value, key=self.name)
        self._value = rs

    def get_ex_item(self, key: str) -> ExItem | None:
        for i in self._coms.values():
            if i.key == key:
                return i

    def get_value(self) -> str:
        v = pretty_dumps(self._value)
        for k, ex in self._coms.items():
            v = v.replace(f'"{k}"', ex.serialize())
        return v

    def get_imports(self) -> imports.ImportDict:
        _imports = imports.merge_imports(
            *(c.get_imports() for c in self._coms.values()),
        )
        return _imports

    def get_state(self) -> str:
        for _, ex in self._coms.items():
            _state = ex.get_state()
            if _state != '':
                return _state
        return ''

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        if version <= '000.004.006':
            hooks = set()
        else:
            hooks = {}
        for _, ex in self._coms.items():
            _hooks = ex.get_hooks()
            hooks.update(_hooks)
        return hooks

    def get_interpolations(self) -> List[Tuple[int, int]]:
        rs = []
        for _, ex in self._coms.items():
            rs += ex.get_interpolations()
        return rs

    def get_var_data(self) -> VarData:
        return VarData(
            state=self.get_state(),
            imports=self.get_imports(),
            hooks=self.get_hooks(),
            interpolations=self.get_interpolations(),
        )


@dataclasses.dataclass(
    eq=False,
    **{"slots": True} if sys.version_info >= (3, 10) else {},
)
class ExVar(BaseVar):
    _var_value: Any = dataclasses.field(default=Any)


class NodeVar(ExVar):
    """
    support:
        . base types: int, float, bool, str
        . js, like: {show ? 11 : 0}
     """
    pass


@dataclasses.dataclass(
    eq=False,
    **{"slots": True} if sys.version_info >= (3, 10) else {},
)
class ContainVar(ExVar):
    _var_fmt: ExFormatter = dataclasses.field(default=None)

    @property
    def _var_full_name(self) -> str:
        return self._var_name

    @classmethod
    def create(cls, _args_: Any = None, **kwargs) -> Self:
        value = _args_ if _args_ is not None else kwargs
        # v: BaseVar = super().create(_name, _var_is_local=_var_is_local, _var_is_string=_var_is_string)
        return cls(
            _var_name='',
            _var_type=cls,
            _var_is_local=True,
            _var_is_string=False,
            _var_data=None,
            _var_value=value,
        )

    def init(self, parent: Component, name: str) -> Self:
        fmt = ExFormatter(self._var_value, parent=parent, name=name)
        self._var_fmt = fmt
        self._var_name = fmt.get_value()
        self._var_data = fmt.get_var_data()
        return self

    def get_custom_components(self) -> set[CustomComponent]:
        rs = set()
        for ex in self._var_fmt._coms.values():
            rs |= ex.get_custom_components()
        return rs

    def get_custom_code(self) -> Set[str]:
        rs = set()
        for ex in self._var_fmt._coms.values():
            rs |= ex.get_custom_code()
        return rs

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        return self._var_data.hooks

    def get_imports(self) -> imports.ImportDict:
        return self._var_data.imports


contain = ContainVar.create


def container(data: Union[list, dict], name: str = '') -> rx.Component:
    """ wrap contain to Component, use for:
        .foreach:  rx.foreach(GlobalState.subnav_items, _sub_item)
            GlobalState.subnav_items = [{"title": "home", "href": "/home"}, ]
            def _sub_item(i: dict) -> rx.Component:
                b = helper.container(
                    dict(title=rx.link(i['title'], href=i['href']))
                )
                return b
    """
    d = contain(data)
    b = Bare()
    d.init(b, name)
    b.contents = d
    return b


# no use
class VarDataMixin:
    def __iter__(self):
        v = Var()
        v._var_data = VarData(
            imports=self.get_imports(),
        )
        yield from [v]


class AntdBaseComponent(Component):
    _custom_components: Set[CustomComponent] = pydantic.PrivateAttr(default_factory=set)

    def __init__(self, *args, **kwargs):
        contains = {}
        exs = {}
        kw = {}
        for k, v in kwargs.items():
            if isinstance(v, ContainVar):
                contains[k] = v
            elif isinstance(v, (JsValue, JsEvent)):
                exs[k] = v
            else:
                kw[k] = v
        # super().__init__(*args, **kw)
        try:
            super().__init__(*args, **kw)
        except Exception as err:
            print(f"class<{self}>, args={args}, kw={kw}, error: {err}")
            raise
        self._init_contains(contains, exs)

    def _get_all_custom_components(
            self, seen: set[str] | None = None
    ) -> Set[CustomComponent]:
        _coms = super()._get_all_custom_components(seen=seen)
        if hasattr(self, '_custom_components') and self._custom_components:
            _coms |= self._custom_components
        return _coms

    def _get_all_custom_code(self) -> Set[str]:
        code = super()._get_all_custom_code()
        for k, v in self._iter_contains():
            code.update(v.get_custom_code())
        return code

    def _get_style(self) -> dict:
        """Get the style for the component.

        Returns:
            The dictionary of the component style as value and the style notation as key.
        """
        return {"style": self.style}

    def _get_events_hooks(self) -> set[str] | Dict[str, None]:
        _hooks = super()._get_events_hooks()
        js_events: List[BaseVar] = [
            v for k, v in self.event_triggers.items()
            if isinstance(v, BaseVar) and v._var_data is not None and v._var_data.hooks]

        if version <= '000.004.006':
            rs = OrderedSet(_hooks)
        else:
            rs = _hooks

        if js_events:
            for ev in js_events:
                rs |= ev._var_data.hooks
        return rs

    def _get_hooks_internal(self) -> Set[str] | Dict[str, None]:
        """Get the React hooks for this component managed by the framework.

        Downstream components should NOT override this method to avoid breaking
        framework functionality.

        Returns:
            Set of internally managed hooks.
        """
        # need order hooks, useContext code need first
        if version <= '000.004.006':
            s = OrderedSet(
                hook
                for hook in [self._get_mount_lifecycle_hook(), self._get_ref_hook()]
                if hook
            )

            s |= self._get_events_hooks()
            var_hooks = self._get_vars_hooks()
            if [h for h in var_hooks if 'addEvents' in h]:
                s.add(Hooks.EVENTS)
            s |= var_hooks
            s |= self._get_special_hooks()
            return s
        else:
            s = {
                hook: None
                for hook in [self._get_ref_hook(), self._get_mount_lifecycle_hook()]
                if hook is not None
            }
            var_hooks = self._get_vars_hooks()
            if [h for h in var_hooks if 'addEvents' in h]:
                s[Hooks.EVENTS] = None
            s |= var_hooks
            s |= self._get_events_hooks()
            s |= self._get_special_hooks()
            return s

    def _iter_contains(self) -> Iterable[Tuple[str, ContainVar]]:
        for k in self.get_fields().keys():
            v = getattr(self, k, None)
            if isinstance(v, ContainVar):
                yield k, v

    def _init_contains(self, contains: Dict[str, ContainVar], exs: Dict[str, Any]):
        for k, v in contains.items():
            v.init(self, k)
            self._custom_components |= v.get_custom_components()
            setattr(self, k, v)

        event_keys = self.get_event_triggers().keys()
        for k, v in exs.items():
            if isinstance(v, (JsValue, JsEvent)):
                item = v.get_ex_item(self, k)
            else:
                raise NotImplementedError(f"Unsupported type: {type(v)}")
            self._custom_components |= item.get_custom_components()
            _v = ExVar(
                _var_name=item.serialize(),
                _var_is_local=True,
                _var_data=item.get_var_data(),
                _var_value=v,
            )
            if k in event_keys:
                self.event_triggers[k] = _v
            else:
                setattr(self, k, _v)


class AntdComponent(AntdBaseComponent):
    """A component that wraps an Antd component."""
    library = "antd"

    @staticmethod
    @lru_cache(maxsize=None)
    def _get_app_wrap_components() -> dict[tuple[int, str], Component]:
        from .antd.base import config_provider
        return {
            (40, "AntdProvider"): _config_provider if _config_provider is not None else config_provider(),
        }


class AntdSubComponent(AntdBaseComponent, Component):
    base_tag: str = None

    def _get_imports(self) -> imports.ImportDict:
        _imports = super()._get_imports()
        return {}


_config_provider: Optional[Component] = None


def default_config(provider: Component):
    global _config_provider
    from .antd.base import ConfigProvider
    assert isinstance(provider, ConfigProvider)
    _config_provider = provider


def patch_all():
    from reflex import constants, vars
    from reflex.compiler import templates

    constants.Templates.Dirs.JINJA_TEMPLATE = [path.join(template_path, 'jinja'),
                                               constants.Templates.Dirs.JINJA_TEMPLATE]

    templates.DOCUMENT_ROOT = templates.get_template("web/pages/_document.js.jinja2")
    old_extract_var_data = vars._extract_var_data

    def _my_extract_var_data(value: Union[Iterable, Component]) -> list[VarData | None]:
        if isinstance(value, Component):
            return [
                VarData(
                    imports=get_component_all_imports(value),
                    hooks=get_component_hooks(value),
                )
            ]
        var_datas = old_extract_var_data(value)
        return var_datas

    vars._extract_var_data = _my_extract_var_data

    def _my_get_memoized_event_triggers(
            cls,
            component: Component,
    ) -> dict[str, tuple[Var, str]]:
        """ fix: some js function can not memo """
        pass

    # StatefulComponent._get_memoized_event_triggers = classmethod(_my_get_memoized_event_triggers)


ReactNode = Union[str, Component]
JsNode = Union[JsValue, JsEvent]




from typing import Optional, Union, Dict, Any, List

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import StatusType


class Alert(AntdComponent):
    tag = 'Alert'

    action: Optional[Var[ReactNode]]
    banner: Optional[Var[bool]]
    closable: Optional[Var[Union[bool, dict]]]
    description: Optional[ReactNode]
    icon: Optional[ReactNode]
    message: Optional[ReactNode]
    show_icon: Optional[bool]
    type: Optional[StatusType]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            "on_close": lambda event: [],
            "after_close": lambda: [],
        })
        return _triggers


class AlertErrorBoundary(AntdComponent):
    tag = 'Alert.ErrorBoundary'

    description: Optional[ReactNode]
    message: Optional[ReactNode]


alert = Alert.create
alert_error_boundary = AlertErrorBoundary.create



from .antd.layout import layout, header, content, footer, sider  # noqa
from .antd.divider import divider  # noqa
from .antd.flex import flex  # noqa
from .antd.grid import row, col  # noqa
from .antd.space import space, space_compact  # noqa



from typing import Any, Dict, Literal, Optional, Union, overload
from reflex.vars import Var, BaseVar, ComputedVar
from reflex.event import EventChain, EventHandler, EventSpec
from reflex.style import Style
from typing import Optional, Union, Dict, Any, List, Tuple
from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers
from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import DatePickerModeType, DatePickerType, SizeType, PlacementType, StatusType, VariantType
from .base import Locale

class DayJS(JsValue):

    def serialize(self) -> str:
        ...

class BaseDatePicker(AntdComponent):

    def get_event_triggers(self) -> Dict[str, Any]:
        ...

    @overload
    @classmethod
    def create(cls, *children, allow_clear: Optional[Union[Var[bool], bool]]=None, auto_focus: Optional[Union[Var[bool], bool]]=None, date_render: Optional[Union[Var[JsValue], JsValue]]=None, cell_render: Optional[Union[Var[JsValue], JsValue]]=None, components: Optional[Union[Var[Union[str, Component, list, dict, ContainVar]], Union[str, Component, list, dict, ContainVar]]]=None, disabled: Optional[Union[Var[bool], bool]]=None, disabled_date: Optional[Union[Var[JsValue], JsValue]]=None, format: Optional[Union[Var[ContainVar], ContainVar]]=None, order: Optional[Union[Var[bool], bool]]=None, popup_class_name: Optional[Union[Var[str], str]]=None, preserve_invalidOn_blur: Optional[Union[Var[bool], bool]]=None, getPopup_container: Optional[Union[Var[JsValue], JsValue]]=None, input_readOnly: Optional[Union[Var[bool], bool]]=None, locale: Optional[Union[Var[Locale], Locale]]=None, min_date: Optional[Union[Var[JsValue], JsValue]]=None, max_date: Optional[Union[Var[JsValue], JsValue]]=None, mode: Optional[Union[Var[Literal['time', 'date', 'month', 'year', 'decade']], Literal['time', 'date', 'month', 'year', 'decade']]]=None, need_confirm: Optional[Union[Var[bool], bool]]=None, next_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, open: Optional[Union[Var[bool], bool]]=None, panel_render: Optional[Union[Var[JsValue], JsValue]]=None, picker: Optional[Union[Var[Literal['date', 'week', 'month', 'quarter', 'year']], Literal['date', 'week', 'month', 'quarter', 'year']]]=None, placeholder: Optional[Union[Var[Union[str, List[str]]], Union[str, List[str]]]]=None, placement: Optional[Union[Var[Literal['start', 'end', 'bottom', 'bottomLeft', 'bottomRight', 'top', 'topLeft', 'topRight']], Literal['start', 'end', 'bottom', 'bottomLeft', 'bottomRight', 'top', 'topLeft', 'topRight']]]=None, popup_style: Optional[Union[Var[ContainVar], ContainVar]]=None, presets: Optional[Union[Var[ContainVar], ContainVar]]=None, prev_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, size: Optional[Union[Var[Literal['default', 'small', 'medium', 'middle', 'large']], Literal['default', 'small', 'medium', 'middle', 'large']]]=None, status: Optional[Union[Var[Literal['default', 'success', 'error', 'warning', 'info', 'exception', 'normal', 'active']], Literal['default', 'success', 'error', 'warning', 'info', 'exception', 'normal', 'active']]]=None, suffix_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, super_next_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, super_prev_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, variant: Optional[Union[Var[Literal['outlined', 'borderless', 'filled']], Literal['outlined', 'borderless', 'filled']]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_open_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_panel_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'BaseDatePicker':
        """Create the component.

        Args:
            *children: The children of the component.
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...

class DatePicker(BaseDatePicker):

    def get_event_triggers(self) -> Dict[str, Any]:
        ...

    @overload
    @classmethod
    def create(cls, *children, default_picker_value: Optional[Union[Var[JsValue], JsValue]]=None, default_value: Optional[Union[Var[ContainVar], ContainVar]]=None, disabled_time: Optional[Union[Var[JsValue], JsValue]]=None, format: Optional[Union[Var[ContainVar], ContainVar]]=None, multiple: Optional[Union[Var[bool], bool]]=None, picker_value: Optional[Union[Var[ContainVar], ContainVar]]=None, render_extra_footer: Optional[Union[Var[JsValue], JsValue]]=None, show_now: Optional[Union[Var[bool], bool]]=None, show_time: Optional[Union[Var[Union[bool, ContainVar]], Union[bool, ContainVar]]]=None, show_week: Optional[Union[Var[bool], bool]]=None, value: Optional[Union[Var[ContainVar], ContainVar]]=None, allow_clear: Optional[Union[Var[bool], bool]]=None, auto_focus: Optional[Union[Var[bool], bool]]=None, date_render: Optional[Union[Var[JsValue], JsValue]]=None, cell_render: Optional[Union[Var[JsValue], JsValue]]=None, components: Optional[Union[Var[Union[str, Component, list, dict, ContainVar]], Union[str, Component, list, dict, ContainVar]]]=None, disabled: Optional[Union[Var[bool], bool]]=None, disabled_date: Optional[Union[Var[JsValue], JsValue]]=None, order: Optional[Union[Var[bool], bool]]=None, popup_class_name: Optional[Union[Var[str], str]]=None, preserve_invalidOn_blur: Optional[Union[Var[bool], bool]]=None, getPopup_container: Optional[Union[Var[JsValue], JsValue]]=None, input_readOnly: Optional[Union[Var[bool], bool]]=None, locale: Optional[Union[Var[Locale], Locale]]=None, min_date: Optional[Union[Var[JsValue], JsValue]]=None, max_date: Optional[Union[Var[JsValue], JsValue]]=None, mode: Optional[Union[Var[Literal['time', 'date', 'month', 'year', 'decade']], Literal['time', 'date', 'month', 'year', 'decade']]]=None, need_confirm: Optional[Union[Var[bool], bool]]=None, next_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, open: Optional[Union[Var[bool], bool]]=None, panel_render: Optional[Union[Var[JsValue], JsValue]]=None, picker: Optional[Union[Var[Literal['date', 'week', 'month', 'quarter', 'year']], Literal['date', 'week', 'month', 'quarter', 'year']]]=None, placeholder: Optional[Union[Var[Union[str, List[str]]], Union[str, List[str]]]]=None, placement: Optional[Union[Var[Literal['start', 'end', 'bottom', 'bottomLeft', 'bottomRight', 'top', 'topLeft', 'topRight']], Literal['start', 'end', 'bottom', 'bottomLeft', 'bottomRight', 'top', 'topLeft', 'topRight']]]=None, popup_style: Optional[Union[Var[ContainVar], ContainVar]]=None, presets: Optional[Union[Var[ContainVar], ContainVar]]=None, prev_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, size: Optional[Union[Var[Literal['default', 'small', 'medium', 'middle', 'large']], Literal['default', 'small', 'medium', 'middle', 'large']]]=None, status: Optional[Union[Var[Literal['default', 'success', 'error', 'warning', 'info', 'exception', 'normal', 'active']], Literal['default', 'success', 'error', 'warning', 'info', 'exception', 'normal', 'active']]]=None, suffix_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, super_next_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, super_prev_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, variant: Optional[Union[Var[Literal['outlined', 'borderless', 'filled']], Literal['outlined', 'borderless', 'filled']]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_ok: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_open_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_panel_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'DatePicker':
        """Create the component.

        Args:
            *children: The children of the component.
            show_week: showTime.defaultValue
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...

class RangePicker(BaseDatePicker):

    def get_event_triggers(self) -> Dict[str, Any]:
        ...

    @overload
    @classmethod
    def create(cls, *children, allow_empty: Optional[Union[Var[Tuple[bool, bool]], Tuple[bool, bool]]]=None, cell_render: Optional[Union[Var[JsValue], JsValue]]=None, date_render: Optional[Union[Var[JsValue], JsValue]]=None, default_picker_value: Optional[Union[Var[ContainVar], ContainVar]]=None, default_value: Optional[Union[Var[ContainVar], ContainVar]]=None, disabled: Optional[Union[Var[Tuple[bool, bool]], Tuple[bool, bool]]]=None, disabled_time: Optional[Union[Var[JsValue], JsValue]]=None, format: Optional[Union[Var[ContainVar], ContainVar]]=None, id: Optional[Union[Var[Dict[str, str]], Dict[str, str]]]=None, picker_value: Optional[Union[Var[ContainVar], ContainVar]]=None, presets: Optional[Union[Var[ContainVar], ContainVar]]=None, render_extra_footer: Optional[Union[Var[JsValue], JsValue]]=None, separator: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, show_time: Optional[Union[Var[Union[bool, ContainVar]], Union[bool, ContainVar]]]=None, value: Optional[Union[Var[ContainVar], ContainVar]]=None, allow_clear: Optional[Union[Var[bool], bool]]=None, auto_focus: Optional[Union[Var[bool], bool]]=None, components: Optional[Union[Var[Union[str, Component, list, dict, ContainVar]], Union[str, Component, list, dict, ContainVar]]]=None, disabled_date: Optional[Union[Var[JsValue], JsValue]]=None, order: Optional[Union[Var[bool], bool]]=None, popup_class_name: Optional[Union[Var[str], str]]=None, preserve_invalidOn_blur: Optional[Union[Var[bool], bool]]=None, getPopup_container: Optional[Union[Var[JsValue], JsValue]]=None, input_readOnly: Optional[Union[Var[bool], bool]]=None, locale: Optional[Union[Var[Locale], Locale]]=None, min_date: Optional[Union[Var[JsValue], JsValue]]=None, max_date: Optional[Union[Var[JsValue], JsValue]]=None, mode: Optional[Union[Var[Literal['time', 'date', 'month', 'year', 'decade']], Literal['time', 'date', 'month', 'year', 'decade']]]=None, need_confirm: Optional[Union[Var[bool], bool]]=None, next_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, open: Optional[Union[Var[bool], bool]]=None, panel_render: Optional[Union[Var[JsValue], JsValue]]=None, picker: Optional[Union[Var[Literal['date', 'week', 'month', 'quarter', 'year']], Literal['date', 'week', 'month', 'quarter', 'year']]]=None, placeholder: Optional[Union[Var[Union[str, List[str]]], Union[str, List[str]]]]=None, placement: Optional[Union[Var[Literal['start', 'end', 'bottom', 'bottomLeft', 'bottomRight', 'top', 'topLeft', 'topRight']], Literal['start', 'end', 'bottom', 'bottomLeft', 'bottomRight', 'top', 'topLeft', 'topRight']]]=None, popup_style: Optional[Union[Var[ContainVar], ContainVar]]=None, prev_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, size: Optional[Union[Var[Literal['default', 'small', 'medium', 'middle', 'large']], Literal['default', 'small', 'medium', 'middle', 'large']]]=None, status: Optional[Union[Var[Literal['default', 'success', 'error', 'warning', 'info', 'exception', 'normal', 'active']], Literal['default', 'success', 'error', 'warning', 'info', 'exception', 'normal', 'active']]]=None, suffix_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, super_next_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, super_prev_icon: Optional[Union[Var[Union[str, Component]], Union[str, Component]]]=None, variant: Optional[Union[Var[Literal['outlined', 'borderless', 'filled']], Literal['outlined', 'borderless', 'filled']]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_calendar_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_open_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_panel_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'RangePicker':
        """Create the component.

        Args:
            *children: The children of the component.
            value: showTime.defaultValue
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...
dayjs = DayJS
date_picker = DatePicker.create
range_picker = RangePicker.create


from typing import Optional, Union, Dict, Any, List, Set
import uuid

from reflex import Component, Var
from reflex.utils import imports

from ..base import AntdComponent, ContainVar, JsValue, ReactNode, ExStateItem, FakeComponentMixin
from ..constant import TypeType

from . import helper


class Modal(AntdComponent):
    tag = 'Modal'

    class_names: Optional[Var[ContainVar]]
    styles: Optional[Var[ContainVar]]
    cancel_button_props: Optional[Var[ContainVar]]
    cancel_text: Optional[Var[ReactNode]]
    centered: Optional[Var[bool]]
    close_icon: Optional[Var[Union[ReactNode, bool]]]
    confirm_loading: Optional[Var[bool]]
    destroy_on_close: Optional[Var[bool]]
    focus_trigger_after_close: Optional[Var[bool]]
    footer: Optional[Var[ContainVar]]
    force_render: Optional[Var[bool]]
    get_container: Optional[Var[Union[bool, ContainVar]]]
    keyboard: Optional[Var[bool]]
    mask: Optional[Var[bool]]
    mask_closable: Optional[Var[bool]]
    modal_render: Optional[Var[ReactNode]]
    ok_button_props: Optional[Var[ContainVar]]
    ok_text: Optional[Var[ReactNode]]
    ok_type: Optional[Var[TypeType]]
    title: Optional[Var[ReactNode]]
    open: Optional[Var[bool]]
    width: Optional[Var[Union[str, int]]]
    wrapClassName: Optional[Var[str]]
    zIndex: Optional[Var[int]]

    def _get_hooks(self) -> str | None:
        _hooks = []
        # if self.open is not None and 'on_open' in self.event_triggers:
        #     _hooks.append(helper.hook_state(
        #         state_field=self.open,
        #         on_update=f'{str(self.event_triggers["on_open"])}()',
        #     ))
        return '\n'.join(_hooks) if _hooks else None

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            "after_close": lambda: [],
            "on_cancel": lambda e: [],
            "on_ok": lambda e: [],
            "after_open_change": lambda open: [open],
        })
        return _triggers


class Confirm(FakeComponentMixin, JsValue):
    config: Optional[Var[dict]]

    @property
    def config_item(self) -> Optional[ExStateItem]:
        if not hasattr(self, 'config') or self.config is None:
            return None
        try:
            return self._config_item
        except AttributeError:
            if isinstance(self.config, Var):
                self._config_item = ExStateItem(self.config, self)
            else:
                self._config_item = ContainVar.create(self.config).init(self, 'config')
            return self._config_item

    @property
    def uid(self) -> str:
        try:
            return self._uid
        except AttributeError:
            self._uid = uuid.uuid4().hex
            return self._uid

    def get_name(self) -> str:
        return f'confirm_{self.uid}'

    def get_imports(self) -> imports.ImportDict:
        _imports = {
            "antd": [imports.ImportVar(tag='Modal')],
        }
        return imports.merge_imports(
            _imports,
            self.config_item.get_imports(),
        )

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        before_open = None
        if isinstance(self.config_item, ContainVar):
            before_open = self.config_item._var_fmt.get_ex_item('config.before_open')
        call_before_open = f'({before_open.serialize()})();' \
            if before_open is not None else ''
        confirm_func = """const %(name)s = () => {
                                %(call_before_open)s
                               Modal.confirm(%(cfg)s);
                               };""" % dict(
            name=self.get_name(),
            call_before_open=call_before_open,
            cfg=self.config_item.serialize() if hasattr(self.config_item, 'serialize') else str(self.config_item),
        )
        return {
            **self.config_item.get_hooks(),
            confirm_func: None,
        }

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = {}
        _triggers.update({
            "config.after_close": lambda: [],
            "config.on_cancel": lambda e: [],
            "config.on_ok": lambda e: [],
            "config.before_open": lambda e: [],
            # not support: "config.after_open_change": lambda open: [open],
        })
        return _triggers

    def serialize(self) -> str:
        return self.get_name()


modal = Modal.create
confirm = Confirm



from typing import Optional, Union, Dict
from types import SimpleNamespace
from functools import lru_cache

import reflex as rx
from reflex import Var, Component
from reflex.vars import BaseVar, VarData
from reflex.utils import imports


from .. import base
from ..base import AntdComponent, ContainVar, JsValue
from ..constant import SizeType


next_theme_var_data = VarData(  # type: ignore
    imports={
        f"next-themes": {imports.ImportVar(tag="useTheme")},
        "react": {imports.ImportVar(tag="useContext")},
        "antd": {imports.ImportVar(tag="theme", alias="AntdTheme")}
    },
    hooks={
        f"const nextTheme = useTheme()": None,
    },
)
next_theme_var = BaseVar(
    _var_name='nextTheme.theme',
    _var_type="str",
    _var_data=next_theme_var_data,
)
light_theme_var = BaseVar(
    _var_name='AntdTheme.defaultAlgorithm',
)
dark_theme_var = BaseVar(
    _var_name='AntdTheme.darkAlgorithm',
)


def theme(**kwargs):
    if 'algorithm' not in kwargs:
        kwargs['algorithm'] = rx.cond(next_theme_var == 'dark', dark_theme_var, light_theme_var)
    return ContainVar.create(**kwargs)


class Locale(JsValue):

    @property
    def lang(self) -> str:
        return self.value.replace("_", "")

    def serialize(self) -> str:
        return self.lang

    def get_imports(self) -> imports.ImportDict:
        return {
            f'antd/locale/{self.value}': [imports.ImportVar(tag=self.lang, install=False, is_default=True)],
        }


class ConfigProvider(AntdComponent):
    """Top level antd provider must be included in any app using antd components."""

    tag = "ConfigProvider"
    alias = "AntdConfigProvider"

    auto_insert_space_in_button: Optional[Var[bool]]
    component_disabled: Optional[Var[bool]]
    component_size: Optional[Var[SizeType]]
    csp: Optional[Var[ContainVar]]
    direction: Optional[Var[str]]
    get_popup_container: Optional[Var[JsValue]]
    get_target_container: Optional[Var[JsValue]]
    icon_prefix_cls: Optional[Var[str]]
    locale: Optional[Var[Locale]]
    popup_match_select_width: Optional[Var[Union[bool, int]]]
    popup_overflow: Optional[Var[str]]
    prefix_cls: Optional[Var[str]]
    render_empty: Optional[Var[JsValue]]
    theme: Optional[Var[Union[Dict, ContainVar]]]
    virtual: Optional[Var[bool]]
    warning: Optional[Var[Dict]]

    @classmethod
    def create(cls, *children, **props) -> Component:
        """Create a new ConfigProvider component.

        Returns:
            A new ConfigProvider component.
        """
        if 'theme' not in props:
            theme = Var.create('theme.styles.global.body.antd', _var_is_local=False)
            props['theme'] = theme

        return super().create(
            *children,
            **props
        )

    def _get_imports(self) -> imports.ImportDict:
        _imports = super()._get_imports()
        _imports.setdefault("/utils/theme.js", []).append(
            imports.ImportVar(tag="theme", is_default=True),
        )
        _imports.setdefault("@ant-design/cssinjs", []).append(
            imports.ImportVar(tag="Theme", alias='antdTheme'),
        )
        return _imports

    @staticmethod
    @lru_cache(maxsize=None)
    def _get_app_wrap_components() -> dict[tuple[int, str], Component]:
        """ support app router """
        if base.APP_ROUTER:
            return {
                (170, "AntdRegistryProvider"): antd_registry_provider(),
            }
        else:
            return {}


class AntdRegistryProvider(Component):
    library = "@ant-design/nextjs-registry"
    tag = "AntdRegistry"


config_provider = ConfigProvider.create
antd_registry_provider = AntdRegistryProvider.create




from typing import Optional, Union, Dict, Any, List

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue, ReactNode


class Empty(AntdComponent):
    tag = 'Empty'

    description: Optional[Var[ReactNode]]
    image: Optional[Var[ReactNode]]
    image_style: Optional[Var[dict]]


empty = Empty.create



from typing import Optional, Union, Dict, Any, List
from reflex import Var, Component
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue,ReactNode
from ..constant import ColorFormatType, SizeType, PlacementType, TriggerType


class ColorPicker(AntdComponent):
    tag = 'ColorPicker'

    allow_clear: Optional[Var[bool]]
    arrow: Optional[Var[bool]]
    default_value: Optional[Var[str]]
    default_format: Optional[Var[str]]
    disabled: Optional[Var[bool]]
    disabled_alpha: Optional[Var[bool]]
    destroy_tooltip_on_hide: Optional[Var[bool]]
    format: Optional[Var[ColorFormatType]]
    open: Optional[Var[bool]]
    presets: Optional[Var[ContainVar]]
    placement: Optional[Var[PlacementType]]
    panel_render: Optional[Var[JsValue]]
    show_text: Optional[Var[Union[bool, JsValue]]]
    size: Optional[Var[SizeType]]
    trigger: Optional[Var[TriggerType]]
    value: Optional[Var[str]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CHANGE: lambda value, hex: [value, hex],
            'on_change_complete': lambda value: [value],
            'on_format_change': lambda format: [format],
            EventTriggers.ON_OPEN_CHANGE: lambda open: [open],
            'on_clear': lambda: [],
        })
        return _triggers


color_picker = ColorPicker.create






from typing import Optional, Union, Dict, Any, List, Tuple
from dataclasses import dataclass

from reflex import Component, Var, Base
from reflex.utils import imports
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import SizeType, PlacementType, TabsType


@dataclass(frozen=True)
class TabItem:
    close_icon: Optional[Var[ReactNode]] = None
    destroy_inactive_tab_pane: Optional[Var[bool]] = None
    disabled: Optional[Var[bool]] = None
    force_render: Optional[Var[bool]] = None
    key: Optional[Union[str, Var[str]]] = None
    label: Optional[Union[ReactNode, Var[ReactNode]]] = None
    icon: Optional[Var[ReactNode]] = None
    children: Optional[Union[ReactNode, Var[ReactNode]]] = None
    closable: Optional[Var[bool]] = None


class Tabs(AntdComponent):
    tag = 'Tabs'

    active_key: Optional[Var[str]]
    add_icon: Optional[Var[ReactNode]]
    animated: Optional[Var[Union[bool, Dict]]]
    centered: Optional[Var[bool]]
    default_active_key: Optional[Var[str]]
    hide_add: Optional[Var[bool]]
    indicator: Optional[Var[Union[Dict, ContainVar]]]
    items: Optional[Var[Union[ContainVar, List]]]
    more_icon: Optional[Var[ReactNode]]
    remove_icon: Optional[Var[ReactNode]]
    popup_class_name: Optional[Var[str]]
    render_tab_bar: Optional[Var[JsValue]]
    size: Optional[Var[SizeType]]
    tab_bar_extra_content: Optional[Var[Union[ReactNode, ContainVar]]]
    tab_bar_gutter: Optional[Var[int]]
    tab_bar_style: Optional[Var[Dict]]
    tab_position: Optional[Var[PlacementType]]
    destroy_inactive_tab_pane: Optional[Var[bool]]
    type: Optional[Var[TabsType]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CHANGE: lambda key: [key],
            EventTriggers.ON_EDIT: lambda e0, e1: [e0, e1],
            "on_tab_click": lambda key, ev: [key, ev],
            "on_tab_scroll": lambda ev: [ev],
        })
        return _triggers


tabs = Tabs.create
tab_item = TabItem




from typing import Optional, Union, Dict, Any, List, Tuple

from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue
from ..constant import TypographyTextType


class TypographyBase(AntdComponent):
    code: Optional[Var[bool]]
    copyable: Optional[Var[Union[bool, ContainVar]]]
    delete: Optional[Var[bool]]
    disabled: Optional[Var[bool]]
    editable: Optional[Var[Union[bool, ContainVar]]]
    ellipsis: Optional[Var[Union[bool, ContainVar]]]

    mark: Optional[Var[bool]]
    italic: Optional[Var[bool]]
    type: Optional[Var[TypographyTextType]]
    underline: Optional[Var[bool]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CLICK: lambda ev: [ev],
            'copyable.on_copy': lambda ev: [ev],
            'editable.on_change': lambda s: [s],
            'editable.on_cancel': lambda: [],
            'editable.on_start': lambda: [],
            'editable.on_end': lambda: [],
            'ellipsis.on_expand': lambda ev, info: [ev, info],
            'ellipsis.on_ellipsis': lambda ellipsis: [ellipsis],
        })
        return _triggers


class TypographyText(TypographyBase):
    tag = 'Typography.Text'

    keyboard: Optional[Var[bool]]
    strong: Optional[Var[bool]]


class TypographyTitle(TypographyBase):
    tag = 'Typography.Title'

    level: Optional[Var[int]]


class TypographyParagraph(TypographyBase):
    tag = 'Typography.Paragraph'

    strong: Optional[Var[bool]]


typography_text = TypographyText.create
typography_title = TypographyTitle.create
typography_paragraph = TypographyParagraph.create




from typing import Optional, Union, Dict, Any

from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import PlacementType, DirectionType


class Dropdown(AntdComponent):
    tag = 'Dropdown'

    arrow: Optional[Var[bool]]
    auto_adjust_overflow: Optional[Var[bool]]
    auto_focus: Optional[Var[bool]]
    disabled: Optional[Var[bool]]
    destroy_popup_on_hide: Optional[Var[bool]]
    dropdown_render: Optional[Var[JsValue]]
    menu: Optional[Var[Union[ContainVar, list]]]
    placement: Optional[Var[PlacementType]]
    trigger: Optional[Var[ContainVar]]
    open: Optional[Var[bool]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()

        _triggers.update({
            EventTriggers.ON_OPEN_CHANGE: lambda open, info: [open, info],
        })
        return _triggers


class DropdownButton(Dropdown):
    tag = 'Dropdown.Button'

    buttons_render: Optional[Var[JsValue]]
    loading: Optional[Var[bool]]
    danger: Optional[Var[bool]]
    icon: Optional[Var[ReactNode]]
    size: Optional[Var[str]]
    type: Optional[Var[str]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()

        _triggers.update({
            EventTriggers.ON_CLICK: lambda: [],
        })
        return _triggers


dropdown = Dropdown.create
dropdown_button = DropdownButton.create



from typing import Optional, Union, Dict, Any

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue
from reflex.constants import EventTriggers
from .base import Locale
from reflex.utils import imports


class DayJS(JsValue):
    def serialize(self) -> str:
        return f"dayjs('{self.value}')"


class Calendar(AntdComponent):
    tag = 'Calendar'

    cell_render: Optional[Var[JsValue]]
    full_cell_render: Optional[Var[JsValue]]
    default_value: Optional[Var[JsValue]]
    disabled_date: Optional[Var[JsValue]]
    fullscreen: Optional[Var[bool]]
    header_render: Optional[Var[JsValue]]
    locale: Optional[Var[Locale]]
    mode: Optional[Var[str]]
    value: Optional[Var[JsValue]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            'on_select': lambda value, mode: [value, mode],
            'on_panel_change': lambda value, mode: [value, mode],
        })
        return _triggers

    def _get_imports(self) -> imports.ImportDict:
        _imports = super()._get_imports()
        _imports.setdefault("dayjs", []).append(
            imports.ImportVar(tag="dayjs", is_default=True, install=False),
        )
        return _imports


dayjs = DayJS
calendar = Calendar.create



from typing import Optional, Union, Dict

from reflex import Component, Var
from reflex.utils import imports

from ..base import AntdComponent, ContainVar
from ..constant import OrientationType, DirectionType


class Row(AntdComponent):
    tag = 'Row'

    align: Optional[Var[str]]
    gutter: Optional[Var[Union[int, ContainVar]]]
    justify: Optional[Var[str]]
    wrap: Optional[Var[str]]


class Col(AntdComponent):
    tag = 'Col'

    flex: Optional[Var[Union[str, int]]]
    offset: Optional[Var[int]]
    order: Optional[Var[int]]
    pull: Optional[Var[int]]
    push: Optional[Var[int]]
    span: Optional[Var[int]]

    xs: Optional[Var[Union[int, Dict]]]
    sm: Optional[Var[Union[int, Dict]]]
    md: Optional[Var[Union[int, Dict]]]
    lg: Optional[Var[Union[int, Dict]]]
    xl: Optional[Var[Union[int, Dict]]]
    xxl: Optional[Var[Union[int, Dict]]]


row = Row.create
col = Col.create



from .antd.badge import badge, badge_ribbon  # noqa
from .antd.table import table  # noqa
from .antd.tree import tree  # noqa
from .antd.avatar import avatar,avatar_group  # noqa
from .antd.calendar import calendar,dayjs  # noqa
from .antd.card import card,card_grid,card_meta  # noqa
from .antd.carousel import carousel  # noqa
from .antd.collapse import collapse  # noqa
from .antd.descriptions import descriptions  # noqa
from .antd.empty import empty  # noqa
from .antd.image import image,image_preview_group  # noqa
from .antd.list import alist,list_item,list_item_meta  # noqa
from .antd.popover import popover  # noqa
from .antd.qrcode import qrcode  # noqa
from .antd.segmented import segmented  # noqa
from .antd.statistic import statistic, statistic_countdown  # noqa
from .antd.tabs import tabs, tab_item  # noqa
from .antd.tag import tag, checkable_tag  # noqa
from .antd.timeline import timeline  # noqa
from .antd.tooltip import tooltip  # noqa
from .antd.tour import tour  # noqa




from typing import Optional, Union, Dict, Any, List

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue,ReactNode
from ..constant import VariantType, SizeType,DirectionType


class Descriptions(AntdComponent):
    tag = 'Descriptions'

    bordered: Optional[Var[bool]]
    colon: Optional[Var[bool]]
    column: Optional[Var[Union[int,dict]]]
    content_style: Optional[Var[Union[dict,ContainVar]]]
    extra: Optional[Var[ReactNode]]
    items: Optional[Var[Union[ContainVar,list]]]
    label_style: Optional[Var[dict]]
    layout: Optional[Var[DirectionType]]
    size: Optional[Var[SizeType]]
    title: Optional[Var[ReactNode]]


descriptions = Descriptions.create



from typing import Optional, Union, Dict, Any, List, Set

from reflex import Component, Var
from reflex.utils import imports
import uuid
from ..base import AntdComponent, ContainVar, JsValue, ReactNode, version
from ..constant import PlacementType, RoleType


class Notification(JsValue):
    btn: Optional[Var[ReactNode]]
    class_name: Optional[Var[str]]
    close_icon: Optional[Var[ReactNode]]
    description: Optional[Var[str]]
    duration: Optional[Var[int]]
    icon: Optional[Var[ReactNode]]
    key: Optional[Var[str]]
    message: Optional[Var[str]]
    placement: Optional[Var[PlacementType]]
    role: Optional[Var[RoleType]]
    props: Optional[Var[Union[dict, ContainVar]]]

    def get_imports(self) -> imports.ImportDict:
        _imports = {
            "antd": [imports.ImportVar(tag='notification')],
        }
        return _imports

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        open_func = """
        const %(name)s = () => {
          notification.open({
            message: '%(message)s',
            description:'%(description)s',
            placement:'%(placement)s',
            duration:'%(duration)s',
          });
        };
        """ % dict(name=self.get_open_notification(), message=self.message, description=self.description,
                   placement=self.placement, duration=self.duration)

        _hooks = []
        _hooks.extend([
            """const [api, contextHolder] = notification.useNotification();""",
            open_func,
        ])
        if version <= '000.004.006':
            return set(_hooks)
        return dict((h, None) for h in _hooks)

    @property
    def uid(self) -> str:
        try:
            return self._uid
        except AttributeError:
            self._uid = uuid.uuid4().hex
            return self._uid

    def get_open_notification(self) -> str:
        return f'openNotification_{self.uid}'

    def serialize(self) -> str:
        return self.get_open_notification()


notification = Notification



from typing import Any, Dict, Literal, Optional, Union, overload
from reflex.vars import Var, BaseVar, ComputedVar
from reflex.event import EventChain, EventHandler, EventSpec
from reflex.style import Style
from typing import Optional, Union, Dict, Any, List
from reflex import Var
from reflex.constants import EventTriggers
from ..base import AntdComponent, ContainVar
from ..constant import StatusType, SizeType

class Checkbox(AntdComponent):

    def get_event_triggers(self) -> Dict[str, Any]:
        ...

    @overload
    @classmethod
    def create(cls, *children, auto_focus: Optional[Union[Var[bool], bool]]=None, checked: Optional[Union[Var[bool], bool]]=None, default_checked: Optional[Union[Var[bool], bool]]=None, disabled: Optional[Union[Var[bool], bool]]=None, indeterminate: Optional[Union[Var[bool], bool]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'Checkbox':
        """Create the component.

        Args:
            *children: The children of the component.
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...

class CheckboxGroup(AntdComponent):

    def get_event_triggers(self) -> Dict[str, Any]:
        ...

    @overload
    @classmethod
    def create(cls, *children, default_value: Optional[Union[Var[List[Union[str, int]]], List[Union[str, int]]]]=None, disabled: Optional[Union[Var[bool], bool]]=None, name: Optional[Union[Var[str], str]]=None, options: Optional[Union[Var[List[Union[str, int, Dict]]], List[Union[str, int, Dict]]]]=None, value: Optional[Union[Var[List[Union[int, str, bool]]], List[Union[int, str, bool]]]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_change: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'CheckboxGroup':
        """Create the component.

        Args:
            *children: The children of the component.
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...
checkbox = Checkbox.create
checkbox_group = CheckboxGroup.create


from typing import Optional, Union, Dict, Any, List

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue, ReactNode


class Image(AntdComponent):
    tag = 'Image'

    alt: Optional[Var[str]]
    fallback: Optional[Var[str]]
    height: Optional[Var[Union[str, int]]]
    placeholder: Optional[Var[ReactNode]]
    description: Optional[Var[ReactNode]]
    preview: Optional[Var[Union[bool, dict]]]
    src: Optional[Var[str]]
    width: Optional[Var[Union[str, int]]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()

        _triggers.update({
            "on_error": lambda event: [],
        })
        return _triggers


class ImagePreviewGroup(AntdComponent):
    tag = 'Image.PreviewGroup'

    preview: Optional[Var[Union[bool, Component]]]
    items: Optional[Var[list]]
    fallback: Optional[Var[str]]


image = Image.create
image_preview_group = ImagePreviewGroup.create



from typing import Literal


ThemeType = Literal["light", "dark"]
PlacementType = Literal["start", "end", "bottom", "bottomLeft", "bottomRight", "top", "topLeft", "topRight"]
AlignType = Literal["start", "end", "center", "baseline", "left", "right"]
OrientationType = Literal["left", "right", "top", "bottom", "center"]
DirectionType = Literal["vertical", "horizontal", "inline"]
SizeType = Literal["default", "small", "medium", "middle", "large"]
StatusType = Literal['default', 'success', 'error', 'warning', 'info', 'exception', 'normal', 'active']
TriggerType = Literal["click", "hover", "focus", 'contextMenu']
VariantType = Literal['outlined', 'borderless', 'filled']
TypeType = Literal['default', 'primary']
RoleType = Literal['alert', 'status']
BreakpointType = Literal['xs', 'sm', 'md', 'lg', 'xl', 'xxl']

ColorFormatType = Literal['rgb', 'hex', 'hsb']

ButtonShape = Literal['default', 'circle', 'round']

TypographyTextType = Literal['secondary', 'success', 'warning', 'danger']

FloatGroupShapeType = Literal["circle", "square"]

BadgeStatusType = Literal["success", 'processing', 'default', 'error', 'warning']

StepsStatusType = Literal['wait', 'process', 'finish', 'error']
StepsType = Literal['default', 'navigation', 'inline']

DatePickerModeType = Literal['time', 'date', 'month', 'year', 'decade']
DatePickerType = Literal['date', 'week', 'month', 'quarter', 'year']

RadioStyleType = Literal['outline', 'solid']
RadioType = Literal['default', 'button']

SelectModeType = Literal['multiple', 'tags']

ProgressType = Literal['line', 'circle', 'dashboard']

TimelineModeType = Literal['left', 'alternate', 'right']

TabsType = Literal['line', 'card', 'editable-card']

QRCodeType = Literal['canvas', 'svg']
QRCodeErrorLevelType = Literal['L', 'M', 'Q', 'H']
QRCodeStatusType = Literal['active', 'expired', 'loading', 'scanned']

MessageType = Literal['info', 'success', 'error', 'loading', 'warning']



from typing import Optional, Union, Dict, Any, List

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue


class Carousel(AntdComponent):
    tag = 'Carousel'

    autoplay: Optional[Var[bool]]
    autoplay_speed: Optional[Var[int]]
    dot_position: Optional[Var[str]]
    dots: Optional[Var[Union[bool, dict]]]
    fade: Optional[Var[bool]]
    infinite: Optional[Var[bool]]
    speed: Optional[Var[int]]
    easing: Optional[Var[str]]
    wait_for_animate: Optional[Var[bool]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            'after_change': lambda current: [],
            'before_change': lambda current, next: [],
        })
        return _triggers


carousel = Carousel.create



from typing import Optional, List, Dict, Any

from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers

from ..base import AntdComponent


class Pagination(AntdComponent):
    tag = 'Pagination'

    current: Optional[Var[int]]
    default_current: Optional[Var[int]]
    default_page_size: Optional[Var[int]]
    disabled: Optional[Var[bool]]
    hide_on_single_page: Optional[Var[bool]]
    page_size: Optional[Var[int]]
    page_size_options: Optional[Var[List[int]]]
    responsive: Optional[Var[bool]]
    show_less_items: Optional[Var[bool]]
    show_quick_jumper: Optional[Var[bool]]
    show_size_changer: Optional[Var[bool]]
    show_title: Optional[Var[bool]]
    # showTotal
    simple: Optional[Var[bool]]
    size: Optional[Var[int]]
    total: Optional[Var[int]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CHANGE: lambda page, page_size: [page, page_size],
            "show_total": lambda total,range: [total,range],
        })
        return _triggers


pagination = Pagination.create



from .antd.progress import progress  # noqa
from .antd.result import result  # noqa
from .antd.skeleton import skeleton  # noqa
from .antd.spin import spin  # noqa
from .antd.watermark import watermark  # noqa
from .antd.alert import alert, alert_error_boundary  # noqa
from .antd.drawer import drawer  # noqa
from .antd.message import message, message_holder  # noqa
from .antd.notification import notification  # noqa
from .antd.modal import modal, confirm  # noqa
from .antd.popconfirm import popconfirm  # noqa



from typing import Optional, Union, Dict, Any

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue, ReactNode


class Avatar(AntdComponent):
    tag = 'Avatar'

    alt: Optional[Var[str]]
    gap: Optional[Var[int]]
    icon: Optional[Var[ReactNode]]
    shape: Optional[Var[str]]
    size: Optional[Var[Union[str, int, Dict]]]
    src: Optional[Var[ReactNode]]
    src_set: Optional[Var[str]]
    draggable: Optional[Var[bool]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            "on_error": lambda: [],
        })
        return _triggers


class AvatarGroup(AntdComponent):
    tag = 'Avatar.Group'

    max_count: Optional[Var[int]]
    max_popover_placement: Optional[Var[str]]
    max_popover_trigger: Optional[Var[str]]
    max_style: Optional[Var[Dict]]
    size: Optional[Var[Union[str, int, Dict]]]
    shape: Optional[Var[str]]


avatar = Avatar.create
avatar_group = AvatarGroup.create



from typing import Optional, Union, Dict, Any, List
from reflex import Var, Component
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue
from ..constant import StatusType, SizeType, RadioStyleType, RadioType


class Rate(AntdComponent):
    tag = 'Rate'

    allow_clear: Optional[Var[bool]]
    allow_half: Optional[Var[bool]]
    auto_focus: Optional[Var[bool]]
    character: Optional[Var[Union[Component, JsValue]]]
    count: Optional[Var[int]]
    default_value: Optional[Var[int]]
    disabled: Optional[Var[bool]]
    tooltips: Optional[Var[List[str]]]
    value: Optional[Var[int]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CHANGE: lambda v: [v],
            "on_hover_change": lambda v: [v],
            EventTriggers.ON_KEY_DOWN: lambda e: [e],
        })
        return _triggers


rate = Rate.create




from typing import Optional, Union, Dict, Any
from reflex import Var, Component

from ..base import AntdComponent, ContainVar, ReactNode


class Breadcrumb(AntdComponent):
    tag = "Breadcrumb"

    params: Optional[Var[dict]]
    items: Optional[Var[Union[ContainVar, list, Component]]]
    separator: Optional[Var[ReactNode]]


class RouteItemType(AntdComponent):
    tag = "RouteItemType"

    class_name: Optional[Var[str]]
    href: Optional[Var[str]]
    path: Optional[Var[str]]
    title: Optional[Var[ReactNode]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            'on_click': lambda e: [e],
        })
        return _triggers


breadcrumb = Breadcrumb.create



from typing import Any, Dict, Literal, Optional, Union, overload
from reflex.vars import Var, BaseVar, ComputedVar
from reflex.event import EventChain, EventHandler, EventSpec
from reflex.style import Style
from typing import Optional, Union, Dict, Any, List, Set, Iterator
import uuid
from reflex import Component, Var
from reflex.utils import imports
from reflex.components.base.bare import Bare
from reflex.vars import BaseVar
from ..base import AntdComponent, ContainVar, JsValue, ReactNode, ExStateItem, version
from ..constant import MessageType
from . import helper

class Message(JsValue):
    is_global: bool
    class_name: Optional[Var[str]]
    content: Optional[Var[ReactNode]]
    duration: Optional[Var[Union[int, float]]]
    icon: Optional[Var[ReactNode]]
    key: Optional[Var[Union[str, int]]]
    style: Optional[Var[dict]]
    type: Optional[MessageType]
    config: Optional[Var[dict]]
    top: Optional[Var[int]]
    rtl: Optional[Var[bool]]
    prefix_cls: Optional[Var[str]]
    max_count: Optional[Var[int]]

    @property
    def config_item(self) -> Optional[ExStateItem]:
        ...

    @property
    def uid(self) -> str:
        ...

    def get_open_message(self) -> str:
        ...

    def get_imports(self) -> imports.ImportDict:
        ...

    def get_hooks(self) -> Set[str] | Dict[str, None]:
        ...

    def serialize(self) -> str:
        ...

class MessageHolder(Bare):

    @overload
    @classmethod
    def create(cls, *children, msg: Optional[Message]=None, contents: Optional[Union[Var[str], str]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'MessageHolder':
        """"""
        ...
message = Message
message_holder = MessageHolder.create


from typing import Any, Dict, Literal, Optional, Union, overload
from reflex.vars import Var, BaseVar, ComputedVar
from reflex.event import EventChain, EventHandler, EventSpec
from reflex.style import Style
from typing import Optional, Union, Dict
from types import SimpleNamespace
from functools import lru_cache
import reflex as rx
from reflex import Var, Component
from reflex.vars import BaseVar, VarData
from reflex.utils import imports
from .. import base
from ..base import AntdComponent, ContainVar, JsValue
from ..constant import SizeType
next_theme_var_data = VarData(imports={f'next-themes': {imports.ImportVar(tag='useTheme')}, 'react': {imports.ImportVar(tag='useContext')}, 'antd': {imports.ImportVar(tag='theme', alias='AntdTheme')}}, hooks={f'const nextTheme = useTheme()': None})
next_theme_var = BaseVar(_var_name='nextTheme.theme', _var_type='str', _var_data=next_theme_var_data)
light_theme_var = BaseVar(_var_name='AntdTheme.defaultAlgorithm')
dark_theme_var = BaseVar(_var_name='AntdTheme.darkAlgorithm')

def theme(**kwargs):
    ...

class Locale(JsValue):

    @property
    def lang(self) -> str:
        ...

    def serialize(self) -> str:
        ...

    def get_imports(self) -> imports.ImportDict:
        ...

class ConfigProvider(AntdComponent):

    @overload
    @classmethod
    def create(cls, *children, auto_insert_space_in_button: Optional[Union[Var[bool], bool]]=None, component_disabled: Optional[Union[Var[bool], bool]]=None, component_size: Optional[Union[Var[Literal['default', 'small', 'medium', 'middle', 'large']], Literal['default', 'small', 'medium', 'middle', 'large']]]=None, csp: Optional[Union[Var[ContainVar], ContainVar]]=None, direction: Optional[Union[Var[str], str]]=None, get_popup_container: Optional[Union[Var[JsValue], JsValue]]=None, get_target_container: Optional[Union[Var[JsValue], JsValue]]=None, icon_prefix_cls: Optional[Union[Var[str], str]]=None, locale: Optional[Union[Var[Locale], Locale]]=None, popup_match_select_width: Optional[Union[Var[Union[bool, int]], Union[bool, int]]]=None, popup_overflow: Optional[Union[Var[str], str]]=None, prefix_cls: Optional[Union[Var[str], str]]=None, render_empty: Optional[Union[Var[JsValue], JsValue]]=None, theme: Optional[Union[Var[Union[Dict, ContainVar]], Union[Dict, ContainVar]]]=None, virtual: Optional[Union[Var[bool], bool]]=None, warning: Optional[Union[Var[Dict], Dict]]=None, _custom_components: Optional[Set[CustomComponent]]=None, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'ConfigProvider':
        """Create a new ConfigProvider component.

        Returns:
            A new ConfigProvider component.
        """
        ...

class AntdRegistryProvider(Component):

    @overload
    @classmethod
    def create(cls, *children, style: Optional[Style]=None, key: Optional[Any]=None, id: Optional[Any]=None, class_name: Optional[Any]=None, autofocus: Optional[bool]=None, custom_attrs: Optional[Dict[str, Union[Var, str]]]=None, on_blur: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_context_menu: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_double_click: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_focus: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_down: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_enter: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_leave: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_move: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_out: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_over: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_mouse_up: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_scroll: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, on_unmount: Optional[Union[EventHandler, EventSpec, list, function, BaseVar]]=None, **props) -> 'AntdRegistryProvider':
        """Create the component.

        Args:
            *children: The children of the component.
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            class_name: The class name for the component.
            autofocus: Whether the component should take the focus once the page is loaded
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The component.
        """
        ...
config_provider = ConfigProvider.create
antd_registry_provider = AntdRegistryProvider.create


from typing import Optional, Union, Dict, Any, List
from reflex import Var, Component
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue


class Upload(AntdComponent):
    tag = 'Upload'

    accept: Optional[Var[str]]
    action: Optional[Var[Union[str, JsValue]]]
    before_upload: Optional[Var[JsValue]]
    custom_request: Optional[Var[JsValue]]
    data: Optional[Var[Union[ContainVar, JsValue]]]
    default_file_list: Optional[Var[List[ContainVar]]]
    directory: Optional[Var[bool]]
    disabled: Optional[Var[bool]]
    file_list: Optional[Var[List[ContainVar]]]
    headers: Optional[Var[Dict]]
    icon_render: Optional[Var[JsValue]]
    is_image_url: Optional[Var[JsValue]]
    list_type: Optional[Var[str]]
    max_count: Optional[Var[int]]
    method: Optional[Var[str]]
    multiple: Optional[Var[bool]]
    name: Optional[Var[str]]
    open_file_dialog_on_click: Optional[Var[bool]]
    preview_file: Optional[Var[JsValue]]
    progress: Optional[Var[Dict]]
    show_upload_list: Optional[Var[Union[bool, Dict]]]
    with_credentials: Optional[Var[Union[bool]]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            EventTriggers.ON_CHANGE: lambda file, file_list: [file, file_list],
            "on_drop": lambda event: [event],
            "on_download": lambda file: [file],
            "on_preview": lambda file: [file],
            "on_remove": lambda file: [file]
        })
        return _triggers


upload = Upload.create




from typing import Optional

from reflex import Component, Var
from reflex.utils import format

from ..base import VarDataMixin


class BaseIconComponent(Component):
    library = "@ant-design/icons"


class IconComponent(BaseIconComponent):
    tag = "None"

    rotate: Optional[Var[int]]
    spin: Optional[Var[bool]]
    two_tone_color: Optional[Var[str]]

    @classmethod
    def create(cls, *children, **props) -> Component:
        """Initialize the Icon component.
        """
        if children:
            if len(children) == 1 and type(children[0]) == str:
                props["tag"] = children[0]
                children = []
            else:
                raise AttributeError(
                    f"Passing multiple children to Icon component is not allowed: remove positional arguments {children[1:]} to fix"
                )
        if "tag" not in props.keys():
            raise AttributeError("Missing 'tag' keyword-argument for Icon")

        # if (
        #     type(props["tag"]) != str
        #     or format.to_snake_case(props["tag"]) not in LUCIDE_ICON_LIST
        # ):
        #     raise ValueError(
        #         f"Invalid icon tag: {props['tag']}. Please use one of the following: {', '.join(LUCIDE_ICON_LIST[0:25])}, ..."
        #         "\nSee full list at https://lucide.dev/icons."
        #     )

        # props["tag"] = format.to_title_case(format.to_snake_case(props["tag"]))
        # props["alias"] = f"antd{props['tag']}"
        return super().create(*children, **props)


icon = IconComponent.create


# class CustomerServiceOutlined(BaseIconComponent):
#     tag = 'CustomerServiceOutlined'
#
#
# class CommentOutlined(BaseIconComponent):
#     tag = 'CommentOutlined'
#
#
# class QuestionCircleOutlined(BaseIconComponent):
#     tag = 'QuestionCircleOutlined'
#
#
# class LaptopOutlined(BaseIconComponent):
#     tag = 'LaptopOutlined'
#
#
# class NotificationOutlined(BaseIconComponent):
#     tag = 'NotificationOutlined'
#
#
# class CloseCircleOutlined(BaseIconComponent):
#     tag = 'CloseCircleOutlined'
#
#
# class CloseSquareOutlined(BaseIconComponent):
#     tag = 'CloseSquareOutlined'
#
#
# class UserOutlined(BaseIconComponent):
#     tag = 'UserOutlined'
